/* rsa.h for openSSL */

#ifndef CYASSL_OPENSSL_RSA
#define CYASSL_OPENSSL_RSA

#define CyaSSL_RSA_GenAdd   wolfSSL_RSA_GenAdd
#define CyaSSL_RSA_LoadDer  wolfSSL_RSA_LoadDer

#include <cyassl/openssl/ssl.h>
#include <wolfssl/openssl/rsa.h>
#endif

